from distutils.core import setup
setup(
  name = 'tuoni',
  packages = ['tuoni'],
  version = '0.5.0',
  license='MIT', 
  description = 'Library to interact with Tuoni C2 API',
  author = 'Shelldot',
  author_email = 'info@shelldot.com',
  url = 'https://github.com/shell-dot/tuoni-pip',
  download_url = 'https://github.com/shell-dot/tuoni-pip/archive/refs/tags/v_05_0.tar.gz',
  keywords = ['tuoni'],
  install_requires=['requests'],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: 3.11',
    'Programming Language :: Python :: 3.12'
  ]
)